@@ -17,9 +17,6 @@ from astropy.convolution import RickerWavelet2DKernel
 
 from photutils.datasets import make_gaussian_sources_image
 
-
-import astropy.units as u
-from astropy.stats.funcs import gaussian_fwhm_to_sigma
 from astropy.convolution import Kernel2D, Gaussian2DKernel
 
 import matplotlib.pyplot as plt
@@ -158,6 +155,7 @@ def test_contmap_init_uncertainty():
     iv_uncertainty = InverseVariance(uncertainty)
     nm = ContMap(data, uncertainty=iv_uncertainty)
 
+
 def test_contmap_compressed():
     data = np.array([1, 2, 3])
     uncertainty = np.array([10, 1, 1], dtype=float)
@@ -710,6 +708,9 @@ def test_contmap_check_SNR(large_map_source):
     # cut for the fit
     npt.assert_allclose(std, 1, rtol=1e-2)
 
+    std, mu = nm.check_SNR(return_mean=True)
+    npt.assert_allclose(std, 1, rtol=1e-2)
+    npt.assert_allclose(mu, 0, atol=1e-2)
 
 # Different Freetype version on travis... 2.8.0 vs 2.6.1
 @pytest.mark.mpl_image_compare(remove_text=True, tolerance=20)
